"""This DCS4COP post-processing module allows to cloud mask which can be used in the GeoTrellis datacube to show the
cloud percentage in the viewer"""
import os
import osr
import gdal
import gdalnumeric
import gdalconst
import numpy as np
from dcs4cop_vmm_postproc.helper import Helper

def cloud2cube(input_path, input_band,
         water_mask_band, water_mask_value,
         external_water_mask_file, external_water_mask_value,
         output_file,
         downSample=False):

    """ Create a cloud mask which can be used in the GeoTrellis datacube to show the cloud percentage in the viewer"""
    logger = Helper.get_logger()
    logger.info('Masking band %s of %s to %s', input_band, input_path, output_file)

    input_file = Helper.build_netcdf_path(input_path, input_band)
    logger.info('Reading %s', input_file)
    input_raster = gdal.Open(input_file, gdal.GA_ReadOnly)
    input_crs = int((input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system']).split(':')[1]) # e.g. 32631
    input_band = input_raster.GetRasterBand(1)
    input_array = gdal.Band.ReadAsArray(input_band)
    #input_array = input_band.ReadAsArray()


    #use external watermask
    logger.info('Reading %s', external_water_mask_file)
    water_mask_raster = gdal.Open(external_water_mask_file, gdal.GA_ReadOnly)
    if water_mask_raster is None:
        logger.error("Failed to open water mask")
        return

    input_res = Helper.get_res(input_raster)

    bbox = Helper.get_bbox(input_raster)
    water_mask_raster = gdal.Warp('', water_mask_raster, format='MEM',
              xRes=input_res[0], yRes=input_res[1],
              outputBounds=[bbox[0], bbox[3], bbox[2], bbox[1]],
              outputBoundsSRS=input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system'],
              dstSRS=input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system'],
              resampleAlg=gdalconst.GRIORA_NearestNeighbour)

    water_mask_band2 = water_mask_raster.GetRasterBand(1)
    water_mask_array2 = gdal.Band.ReadAsArray(water_mask_band2)


    #user netcdf water/land mask
    if water_mask_band is not None:
        water_mask_file = Helper.build_netcdf_path(input_path, water_mask_band)
        logger.info('Reading %s', water_mask_file)
        water_mask_raster = gdal.Open(water_mask_file, gdal.GA_ReadOnly)
        if water_mask_raster is None:
            logger.error("Failed to open water mask")
            return

        water_mask_band = water_mask_raster.GetRasterBand(1)
        water_mask_array1 = gdal.Band.ReadAsArray(water_mask_band)
        #only use water data : value=1
        water_mask_array1 = np.where(water_mask_array1==1,water_mask_array1,0)
        #use 'OR' operator on both land/water mask to get a more complete land/water mask
        water_mask_array_complete = np.logical_or(water_mask_array1, water_mask_array2)

    else:
        water_mask_array_complete = water_mask_array2

    # apply water mask
    # and convert input no_data (0) to 2, input no_cloud (1) to 0 and input cloud (2) to 1 in a single numpy statement
    masked_band = np.where(
        np.logical_and(
            input_array != input_band.GetNoDataValue(),
            water_mask_array_complete == water_mask_value),
        input_array - 1, 2)

    oldmask = os.umask(0o002)
    Helper.create_parent_dirs(output_file)

    tmp_file = os.path.join('.', os.path.basename(output_file))

    driver = gdal.GetDriverByName("GTiff")
    output_raster = driver.Create(tmp_file, input_raster.RasterXSize, input_raster.RasterYSize,
                              1, gdal.GDT_Byte,
                              options=['COMPRESS=LZW', 'TILED=YES'])
    output_raster.SetGeoTransform(input_raster.GetGeoTransform())

    srs = osr.SpatialReference()
    srs.ImportFromEPSG(input_crs)  # set named SRS because mandatory for GeoServer
    output_raster.SetProjection(srs.ExportToWkt())

    output_band = output_raster.GetRasterBand(1)
    output_band.SetNoDataValue(2)
    if input_band.GetScale() is not None:
        output_band.SetScale(input_band.GetScale())
    if input_band.GetOffset() is not None:
        output_band.SetOffset(input_band.GetOffset())
    gdalnumeric.BandWriteArray(output_band, masked_band)

    output_raster.FlushCache()
    # make sure data is written to disk:
    output_raster = None

    if downSample:
        if input_res[0] < 100:  # downsample to 20m resolution to improve datacube response times (for S2 products)
            target_res = 100
        else:  # just write the file (for L8 products) in native resolution
            target_res = input_res[0]
    else:
        target_res = input_res[0]


    tmp_ds = gdal.Open(tmp_file)
    gdal.Translate(output_file, tmp_ds, xRes=target_res, yRes=target_res, creationOptions=['COMPRESS=LZW', 'TILED=YES'])
    tmp_ds = None
    os.umask(oldmask)
