"""Module containing Helper class useful for DCS4COP post processing"""
import os
import re
import glob
import errno
import logging
import gdal

class Dcs4cop_product(object):

    def __init__(self, inFile, productType, outRootDir):
        self.inFile = inFile
        self.inFileName = os.path.basename(self.inFile).split('.')[0]
        #self.bandname = inputBand
        self.productType = productType #s2 or l8
        self.getDateFromS2File()
        self.getTileIDFromS2File()
        self.outFileDict={}
        self.outFileDict['rgb'] = os.path.join(outRootDir,'RGB', self.yyyy,self.mm, self.inFileName + '_RGB.tiff')
        self.outFileDict['spm'] = os.path.join(outRootDir,'SPM', self.yyyy,self.mm, self.inFileName + '_SPM.tiff')
        self.outFileDict['spmScaled'] = os.path.join(outRootDir,'SPM_BYTE', self.yyyy,self.mm, self.inFileName + '_SPM.tiff')
        self.outFileDict['tur'] = os.path.join(outRootDir,'TUR', self.yyyy,self.mm, self.inFileName + '_TUR.tiff')
        self.outFileDict['turScaled'] = os.path.join(outRootDir,'TUR_BYTE', self.yyyy,self.mm, self.inFileName + '_TUR.tiff')
        self.outFileDict['chl'] = os.path.join(outRootDir,'CHL', self.yyyy,self.mm, self.inFileName + '_CHL.tiff')
        self.outFileDict['chlScaled'] = os.path.join(outRootDir,'CHL_BYTE', self.yyyy,self.mm, self.inFileName + '_CHL.tiff')
        self.outFileDict['ndavi'] = os.path.join(outRootDir,'NDAVI', self.yyyy,self.mm, self.inFileName + '_NDAVI.tiff')
        self.outFileDict['ndaviScaled'] = os.path.join(outRootDir,'NDAVI_BYTE', self.yyyy,self.mm, self.inFileName + '_NDAVI.tiff')
        self.outFileDict['cloudmask'] = os.path.join(outRootDir,'CLOUDMASK', self.yyyy,self.mm, self.inFileName + '_CLOUDMASK.tiff')

    def hasNDAVI(self):

        input_file = Helper.build_netcdf_path(self.inFile, 'NDAVI')
        input_raster = gdal.Open(input_file, gdal.GA_ReadOnly)
        try:
            input_band = input_raster.GetRasterBand(1)
        except:
            return False
        return True

    def getDateFromS2File(self):
        pattern='.*([0-9]{4})([0-9]{2})([0-9]{2})T[0-9]{6}Z.*'
        match= re.search(pattern, self.inFile)
        self.yyyy=match.group(1)
        self.mm = match.group(2)
        self.dd = match.group(3)
        self.yyyymmdd = self.yyyy + self.mm + self.dd

    def getTileIDFromS2File(self):
        pattern='.*_([0-9]{2}[A-Z]{3})_.*'
        match= re.search(pattern, self.inFileName)
        self.tileID=match.group(1)


    def outputExist(self):
        """check if one of all oupputs are missing on the filesystem"""
        exist=True

        for ftype, fpath in  self.outFileDict.items():
            empty_file = fpath.replace('.tiff','.empty')
            if (not os.path.exists(fpath) and not os.path.exists(empty_file)):
                print("it does not exist: " + fpath)
                exist=False
        return exist
        

class Helper(object):
    """Helper class for DCS4COP post processing"""
    
    @staticmethod
    def filterIfOutPutExists(dcs4cop_products):
        #filter inputproducts for which an output already exists
        filteredProductLst=[]
        for product in dcs4cop_products:
            if product.outputExist():
                continue
            filteredProductLst.append(product)
            print( 'adding product: '+ product.inFile)
        return filteredProductLst


    @staticmethod
    def get_logger():
        """ Get a logger which is usable in a Spark context """

        logger = logging.getLogger("py4j")
        #logger.setLevel(logging.INFO)
        logger.setLevel(logging.DEBUG)

        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)

        if len(logger.handlers) == 0:
            logger.addHandler(handler)
        else:
            # avoid adding multiple handlers which would cause one message to be printed multiple times
            logger.handlers[0] = handler

        return logger

    @staticmethod
    def get_products(input_root_dir, extension):
        """ Get all input products with the specified extension """

        products = []
        for dirpath, _, filenames in os.walk(input_root_dir):
            for filename in filenames:
                if filename.endswith(extension):
                    products.append(os.path.join(dirpath, filename))
        return products

    @staticmethod
    def build_output_path(input_root_dir, input_path, output_root_dir, product_type=None):
        """ Build output path for file based on path of input file """

        norm_input_root_dir = os.path.normpath(input_root_dir)
        norm_product_dir = os.path.normpath(os.path.dirname(input_path))
        dir_diff = norm_product_dir[len(norm_input_root_dir)+1:]
        return os.path.join(output_root_dir, dir_diff,
                            Helper.build_filename(input_path, product_type))

    @staticmethod
    def build_filename(input_path, product_type=None):
        """ Build output filename based on input path and product type.
        For L8 products the timestamps differ per date.
        In case of multiple products per date, take the first timestamp. """

        from os.path import dirname as up

        folder = up(up(up(os.path.join(os.path.dirname(input_path)))))
        base = os.path.basename(input_path)
        pattern = re.compile('(.*)(\d{8})T(\d{6})(.*)\.(nc|tiff)')
        match = pattern.match(base)

        first_input_path = sorted(glob.glob(os.path.join(folder + '/*/*/*/', match.group(1) + match.group(2)) + 'T*'))[0]

        first_base = os.path.basename(first_input_path) # e.g. DCS4COP_L8T1_0021_FLANDERS_20180304T103331Z_198024_V003.nc
        first_match = pattern.match(first_base)
        if product_type is not None:
            return match.group(1).replace('DCS4COP', 'DCS4COP_' + product_type) \
                   + match.group(2) + 'T' + first_match.group(3) + match.group(4) + '.tiff'
        else:
            return match.group(1) + match.group(2) + 'T' + first_match.group(3) + match.group(4) + '.tiff'

    @staticmethod
    def create_parent_dirs(output_file):
        """ Create parent directories needed for creation of output file """

        try:
            os.makedirs(os.path.dirname(output_file))
        except OSError as ex:
            if ex.errno == errno.EEXIST:
                pass
            else:
                raise

    @staticmethod
    def build_netcdf_path(file_path, band):
        """ Build NetCDF string in order to be usable with GDAL """
        return 'NETCDF:"' + file_path + '":' + band

    @staticmethod
    def get_bbox(src):
        """ Get the bounding box of a raster in the format [ulx uly lrx lry] """
        ulx, xres, _, uly, _, yres  = src.GetGeoTransform()
        lrx = ulx + (src.RasterXSize * xres)
        lry = uly + (src.RasterYSize * yres)
        return [ulx, uly, lrx, lry]

    @staticmethod
    def get_res(src):
        """ Get the resolution of a raster in the format [xres yres] """
        _, xres, _, _, _, yres = src.GetGeoTransform()
        return [xres, yres]
