"""This DCS4COP post-processing module allows to mask a DCS4COP NetCDF product"""
import os
import osr
import gdalconst
import gdal
import numpy as np
from dcs4cop_vmm_postproc.helper import Helper


def mask(input_path, input_band,
         water_mask_band, water_mask_value,
         external_water_mask_file, external_water_mask_value,
         cloud_mask_band, cloud_mask_value,
         output_file):
    """ Mask a NetCDF input product, identified by input_path, a cloud mask and an external water mask"""

    logger = Helper.get_logger()
    logger.info('Masking band %s of %s to %s', input_band, input_path, output_file)

    input_file = Helper.build_netcdf_path(input_path, input_band)
    logger.info('Reading %s', input_file)
    input_raster = gdal.Open(input_file, gdal.GA_ReadOnly)
    input_crs = int((input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system']).split(':')[1]) # e.g. 32631
    input_band = input_raster.GetRasterBand(1)
    input_array = gdal.Band.ReadAsArray(input_band)
    #input_array = input_band.ReadAsArray()

    if cloud_mask_band != None:
        cloud_mask_file = Helper.build_netcdf_path(input_path, cloud_mask_band)
        logger.info('Reading %s', cloud_mask_file)
        cloud_mask_raster = gdal.Open(cloud_mask_file, gdal.GA_ReadOnly)
        if cloud_mask_raster is None:
            logger.error("Failed to open cloud mask")
            return

        cloud_mask_band = cloud_mask_raster.GetRasterBand(1)
        cloud_mask_array = gdal.Band.ReadAsArray(cloud_mask_band)
    else:
        cloud_mask_array = np.ones(input_array.shape)

    logger.info('Reading %s', external_water_mask_file)
    water_mask_raster = gdal.Open(external_water_mask_file, gdal.GA_ReadOnly)
    if water_mask_raster is None:
        logger.error("Failed to open water mask")
        return

    input_res = Helper.get_res(input_raster)

    bbox = Helper.get_bbox(input_raster)
    water_mask_raster = gdal.Warp('', water_mask_raster, format='MEM',
              xRes=input_res[0], yRes=input_res[1],
              outputBounds=[bbox[0], bbox[3], bbox[2], bbox[1]],
              outputBoundsSRS=input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system'],
              dstSRS=input_raster.GetMetadata()['NC_GLOBAL#geographic_coordinate_system'],
              resampleAlg=gdalconst.GRIORA_NearestNeighbour)

    water_mask_band2 = water_mask_raster.GetRasterBand(1)
    water_mask_array2 = gdal.Band.ReadAsArray(water_mask_band2)

    if water_mask_band is not None:
        water_mask_file = Helper.build_netcdf_path(input_path, water_mask_band)
        logger.info('Reading %s', water_mask_file)
        water_mask_raster = gdal.Open(water_mask_file, gdal.GA_ReadOnly)
        if water_mask_raster is None:
            logger.error("Failed to open water mask")
            return

        water_mask_band = water_mask_raster.GetRasterBand(1)
        water_mask_array1 = gdal.Band.ReadAsArray(water_mask_band)
        # only use water data : value=1
        water_mask_array1 = np.where(water_mask_array1 == 1, water_mask_array1, 0)

        #use OR operator on both land/water mask to get a more complete land/water mask
        water_mask_array_complete = np.logical_or(water_mask_array1, water_mask_array2)
    else:
        water_mask_array_complete = water_mask_array2


    masked_band = np.where(
        np.logical_and(
            np.logical_and(cloud_mask_array == cloud_mask_value, input_array != input_band.GetNoDataValue()),
            water_mask_array_complete == water_mask_value),
        input_array, input_band.GetNoDataValue())

    #check if there is only nodata values
    
    if np.all(masked_band == input_band.GetNoDataValue()):
        logger.info('This dataset is empty')
        empty = True
    else:
        empty = False

    oldmask = os.umask(0o002)
    Helper.create_parent_dirs(output_file)

    logger.info('Writing %s', output_file)

    driver = gdal.GetDriverByName("GTiff")
    output_raster = driver.Create(output_file, input_raster.RasterXSize, input_raster.RasterYSize,
                                  1, gdal.GDT_Int16,
                                  options=['COMPRESS=LZW', 'TILED=YES'])
    output_raster.SetGeoTransform(input_raster.GetGeoTransform())

    srs = osr.SpatialReference()
    srs.ImportFromEPSG(input_crs) # set named SRS because mandatory for GeoServer
    output_raster.SetProjection(srs.ExportToWkt())

    output_band = output_raster.GetRasterBand(1)
    in_scale = input_band.GetScale()
    in_offset = input_band.GetOffset()
    output_band.SetNoDataValue(input_band.GetNoDataValue())
    if in_scale is not None:
        output_band.SetScale(in_scale)
    if in_offset is not None:
        output_band.SetOffset(in_offset)
    gdal.Band.WriteArray(output_band, masked_band)

    output_raster.FlushCache()

    os.umask(oldmask)
    masked_band = None
    
    return output_file
