"""This DCS4COP post-processing module allows to scale a masked DCS4COP GeoTIFF product"""
import os
import gdal
import gdalnumeric
import numpy as np
from datetime import datetime
from dcs4cop_vmm_postproc.helper import Helper


def _get_product_date(input_file):

    file_parts = os.path.basename(input_file).split('_')
    return datetime.strptime(file_parts[len(file_parts) - 3][0:8], "%Y%m%d")


def scale_array(input_array, input_nodata, input_scale, input_offset,
                max_phys_value, scale_factor, min_phys_value=0, output_offset=0):


    '''
    PV = DV * input_scale + PV_offset
    DV = (PV - PV_offset)/ input_scale
    '''
    logger = Helper.get_logger()
    # unscale input raster
    logger.debug("unscaling...")
    unscaled_band = np.where(input_array != input_nodata,
                             input_array * input_scale + input_offset,
                             input_nodata)

    input_array = None


    logger.debug("clipping...")
    # set values > max physical value to max physical value
    scaled_band = np.where(
        np.logical_and(unscaled_band != input_nodata, unscaled_band > max_phys_value),
        max_phys_value, unscaled_band)
    # set values < min physical value to min physical value
    scaled_band = np.where(
        np.logical_and(unscaled_band != input_nodata, unscaled_band < min_phys_value),
        min_phys_value, scaled_band)


    unscaled_band = None
    # rescale values <= max physical value
    logger.debug("scaling... with value %s and offset %s" % (scale_factor,output_offset))
    scaled_band = np.where(
        np.logical_and(scaled_band <= max_phys_value, scaled_band != input_nodata),
        (scaled_band - output_offset)/ scale_factor, scaled_band)

    # set correct nodata value
    scaled_band = np.where(
        scaled_band == input_nodata, 255, scaled_band).astype(np.dtype('B'))

    return scaled_band


def scale(input_file, max_phys_value, scale_factor, output_file, min_phys_value=0, output_offset=0):
    """ Scale a masked GeoTIFF input product to Byte"""

    logger = Helper.get_logger()
    logger.debug('Scaling file %s to %s', input_file, output_file)

    logger.info('Reading %s', input_file)

    input_raster = gdal.Open(input_file, gdal.GA_ReadOnly)
    input_band = input_raster.GetRasterBand(1)
    input_array = gdalnumeric.BandReadAsArray(input_band)
    logger.debug('Done reading %s', input_file)
    
    scaled_array = scale_array(input_array, input_band.GetNoDataValue(), input_band.GetScale(), input_band.GetOffset(),
                               max_phys_value, scale_factor,min_phys_value, output_offset)

    input_band = None
    input_array = None
    logger.info('Writing %s', input_file)
    
    
    oldmask = os.umask(0o002)
    Helper.create_parent_dirs(output_file)
    

    driver = gdal.GetDriverByName("GTiff")
    output_raster = driver.Create(output_file, input_raster.RasterXSize, input_raster.RasterYSize,
                                  1, gdal.GDT_Byte, options=['COMPRESS=LZW', 'TILED=YES'])
    output_raster.SetGeoTransform(input_raster.GetGeoTransform())
    output_raster.SetProjection(input_raster.GetProjection())
    output_band = output_raster.GetRasterBand(1)
    output_band.SetNoDataValue(255)
    output_band.SetScale(scale_factor)
    output_band.SetOffset(output_offset)
    gdalnumeric.BandWriteArray(output_band, scaled_array)

    output_raster.FlushCache()
    output_raster = None
    output_band = None
    scaled_array = None

    os.umask(oldmask)
    logger.info('Done SCALING')
